//
//  ViewController.swift
//  DBSCalendar
//
//  Created by Sreekanth PULICHERLA on 4/2/18.
//  Copyright © 2018 Sreekanth PULICHERLA. All rights reserved.
//

import UIKit
import JTAppleCalendar

protocol getDateDelegate {
    func selectedDate(date: String, selected: Date)
}


class ViewController: UIViewController {
    
    let preDateSelectable: Bool = false
    var delegate: getDateDelegate!
    @IBOutlet weak var calendarCollection: JTAppleCalendarView!
    let outsideMonthColor = UIColor(hex: 0x584a66)
    let monthColor = UIColor.red
    let selectedMonthColor = UIColor(hex: 0x3a294b)
    let currentDateSelectedViewColor = UIColor(hex: 0x4e3f5d)
    var selected = Date()
    let formatter = DateFormatter()
    var firstDate : Date?

    override func viewDidLoad() {
        super.viewDidLoad()
        setupCalendarView()
        calendarCollection.register(UINib(nibName: "DateCellHeader", bundle: Bundle.main),
                              forSupplementaryViewOfKind: UICollectionElementKindSectionHeader,
                              withReuseIdentifier: "DateCellHeader")
        calendarCollection.register(UINib(nibName: "DateCelltemplate", bundle: nil), forCellWithReuseIdentifier: "DateCelltemplate")
        calendarCollection.calendarDelegate = self
        calendarCollection.calendarDataSource = self
        // Do any additional setup after loading the view, typically from a nib.
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func setupCalendarView() {
        
        // Setup calendar spacing
        calendarCollection.minimumLineSpacing = 1
        calendarCollection.minimumInteritemSpacing = 0
        //calendarCollection.allowsMultipleSelection  = true
        calendarCollection.isRangeSelectionUsed = true
        // Setup labels
        
        calendarCollection.scrollToDate(selected, animateScroll: false)
        //calendarCollection.selectDates([selected])
        calendarCollection.visibleDates { (visibleDates) in
            self.setupViewsOfCalendar(from: visibleDates)
        }
    }
    
    func setupViewsOfCalendar(from visibleDates: DateSegmentInfo) {
        let date = visibleDates.monthDates.first!.date
        
        formatter.dateFormat = "yyyy"
        let year = formatter.string(from: date)
        
        formatter.dateFormat = "MMMM"
        let month = formatter.string(from: date)
        //monthLabel.text = "\(month), \(year)"
    }

    func setDate(date: Date) {
        let formatters = DateFormatter()
        formatters.dateFormat = "dd MMM yyyy"
        self.selected = date
        
    }
    
    func handleCellTextColor(view: JTAppleCell?, cellState: CellState) {
        guard let validCell = view as? DateCelltemplate else { return }
        if cellState.isSelected {
            validCell.dateLabel.textColor = UIColor.darkGray
        } else {
            
            if cellState.dateBelongsTo == .thisMonth {
                validCell.dateLabel.textColor = monthColor
                
            } else {
                validCell.dateLabel.textColor = outsideMonthColor
            }
        }
    }
    
    func handleDeCellTextColor(view: JTAppleCell?, cellState: CellState,date: Date) {
        guard let validCell = view as? DateCelltemplate else { return }
        
        if (firstDate == date){
            // first date should not be removed
        }else{
            validCell.dateLabel.textColor = UIColor.darkGray
        }
//        if cellState.isSelected {
//            validCell.dateLabel.textColor = UIColor.darkGray
//        } else {
//
//            if cellState.dateBelongsTo == .thisMonth {
//                validCell.dateLabel.textColor = monthColor
//
//            } else {
//                validCell.dateLabel.textColor = outsideMonthColor
//            }
//        }
    }
    
    func handleCellSelected(view: JTAppleCell?, cellState: CellState, date: Date) {
        guard let validCell = view as? DateCelltemplate else { return }
        
        if cellState.dateBelongsTo != .thisMonth {
            validCell.dateLabel.text = ""
            validCell.isUserInteractionEnabled = false
            validCell.dateView.isHidden = true
        } else if date.isSmaller(to: Date()) && !preDateSelectable {
            //validCell.dateLabel.text = "-"
            validCell.isUserInteractionEnabled = false
            validCell.dateView.isHidden = true
        } else {
        
            validCell.isUserInteractionEnabled = true
            if cellState.isSelected {
                view?.transform = CGAffineTransform(scaleX: 0.5, y: 0.5)
                UIView.animate(
                    withDuration: 0.5,
                    delay: 0, usingSpringWithDamping: 0.3,
                    initialSpringVelocity: 0.1,
                    options: UIViewAnimationOptions.beginFromCurrentState,
                    animations: {
                        view?.transform = CGAffineTransform(scaleX: 1, y: 1)
                },
                    completion: { _ in
                })
                
                validCell.dateView.isHidden = false
                validCell.dateView.backgroundColor = UIColor.red
                validCell.dateView.layer.cornerRadius = 20
            } else {
                    validCell.dateView.backgroundColor = UIColor.white
                    validCell.dateView.layer.cornerRadius = 0
                    validCell.dateView.isHidden = true
               
                
            }
            validCell.dateLabel.text = cellState.text
         }
       
    }
    
    func handleDeCellSelected(view: JTAppleCell?, cellState: CellState, date: Date) {
        guard let validCell = view as? DateCelltemplate else { return }
        
        if (firstDate == date){
                print("same dates so ")
        }else{
            validCell.dateView.backgroundColor = UIColor.white
            validCell.dateView.layer.cornerRadius = 0
            validCell.dateView.isHidden = true
        }
        
    }

}

extension ViewController: JTAppleCalendarViewDataSource {
    
    func configureCalendar(_ calendar: JTAppleCalendarView) -> ConfigurationParameters {
        
        formatter.dateFormat = "yyyy MM dd"
        formatter.timeZone = Calendar.current.timeZone
        formatter.locale = Calendar.current.locale
        let startDate = Date()
        let endDate = formatter.date(from: "2018 06 30")
        let parameters = ConfigurationParameters(startDate: startDate, endDate: endDate!)
        return parameters
    }
}

extension ViewController: JTAppleCalendarViewDelegate {
    func calendar(_ calendar: JTAppleCalendarView, willDisplay cell: JTAppleCell, forItemAt date: Date, cellState: CellState, indexPath: IndexPath) {
        //
        let cell = cell as! DateCelltemplate
        cell.dateLabel.text = cellState.text
        handleCellSelected(view: cell, cellState: cellState, date: date)
        handleCellTextColor(view: cell, cellState: cellState)
    }
    
    
    
    // Display Cell
    func calendar(_ calendar: JTAppleCalendarView, cellForItemAt date: Date, cellState: CellState, indexPath: IndexPath) -> JTAppleCell {
        let cell = calendar.dequeueReusableJTAppleCell(withReuseIdentifier: "DateCelltemplate", for: indexPath) as! DateCelltemplate
        cell.dateLabel.text = cellState.text
        
        handleCellSelected(view: cell, cellState: cellState, date: date)
        handleCellTextColor(view: cell, cellState: cellState)
        return cell
    }
    
    func calendar(_ calendar: JTAppleCalendarView, didSelectDate date: Date, cell: JTAppleCell?, cellState: CellState) {
        if firstDate != nil {
            calendarCollection.selectDates(from: firstDate!, to: date,  triggerSelectionDelegate: false, keepSelectionIfMultiSelectionAllowed: true)
        } else {
            firstDate = date
            handleCellSelected(view: cell, cellState: cellState, date: date)
            handleCellTextColor(view: cell, cellState: cellState)
            setDate(date: date)
        }
       
    }
    
    func calendar(_ calendar: JTAppleCalendarView, didDeselectDate date: Date, cell: JTAppleCell?, cellState: CellState) {
        
        handleDeCellSelected(view: cell, cellState: cellState, date: date)
        handleDeCellTextColor(view: cell, cellState: cellState, date: date)
    }
    
    func calendar(_ calendar: JTAppleCalendarView, didScrollToDateSegmentWith visibleDates: DateSegmentInfo) {
        setupViewsOfCalendar(from: visibleDates)
    }
    
    func calendar(_ calendar: JTAppleCalendarView, sectionHeaderSizeFor range: (start: Date, end: Date), belongingTo month: Int) -> CGSize {
        return CGSize(width: 0, height: 40)
    }
    
    func calendarSizeForMonths(_ calendar: JTAppleCalendarView?) -> MonthSize? {
        return MonthSize(defaultSize: 40)
    }
    
    func calendar(_ calendar: JTAppleCalendarView, headerViewForDateRange range: (start: Date, end: Date), at indexPath: IndexPath) -> JTAppleCollectionReusableView {
        let headerCell = calendar.dequeueReusableJTAppleSupplementaryView(withReuseIdentifier: "DateCellHeader", for: indexPath) as! DateCellHeader
        headerCell.dateHeader.text =  "Month Start"
        return headerCell
    }
}

